#include "../Framework/Framework.h"
#include "../Framework/Util/T3dLoader.h"
#include "../Framework/Math/Color.h"
#include "BumpPolygon.h"



class MainApp : public OpenGLApp {
protected:
	TextureID base0, bump0, gloss0;
	//TextureID base1, bump1, gloss1;
	TextureID distanceMap, shadowCubemap, lightTexture;
	ShaderID phong, distance;

    MaterialTable matTable;

	Set <class Polygon *> polys;
	int firstPolygons;


	Vertex lights[2];
	Color cols[2];

	void drawWorld();

public:
	void selectPixelFormat(PixelFormat &pixelFormat);

	bool init();
	bool exit();
	bool load();
	bool unload();
	bool drawFrame();
	
	void resetCamera();
};


void MainApp::resetCamera(){
	position = Vertex(-800, -40, 0);
	Wx = 0;
	Wy = -PI / 2;
	Wz = 0;
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 8;

	pixelFormat.depthBits = 16;
}

bool polygonFilter(const int flags){
	return ((flags & PF_Invisible) == 0) && ((flags & PF_TwoSided) == 0) && (flags >= 0);
}

bool MainApp::init(){

	projection.loadProjectionMatrix(90 * PI / 180, 0.75f, 1, 4000);

	return true;
}

bool MainApp::exit(){

	return true;
}

bool MainApp::load(){

	String supported, missing;

    if (GL_ARB_fragment_program) supported += "GL_ARB_fragment_program\n"; else missing += "GL_ARB_fragment_program\n";
#ifdef _WIN32
	if (WGL_ARB_render_texture)  supported += "WGL_ARB_render_texture\n";  else missing += "WGL_ARB_render_texture\n";
#else
	if (GLX_ATI_render_texture)  supported += "GLX_ATI_render_texture\n";  else missing += "GLX_ATI_render_texture\n";
#endif

    if (missing.getLength() > 0){
        ALERT("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extensions aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions");
        return false;
    }
	
	shadowCubemap = renderer->addRenderTexture(512, 512, false, true, RPF_RGBA8);
	
	phong = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "Phong.fsh");

	distance = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "Distance.fsh");

	if (glGetError() != GL_NO_ERROR){
		char *str = (char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB);
		ALERT(str, "Error");
	}

	if (glGetError() != GL_NO_ERROR){
		char *str = (char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB);
		ALERT(str, "Error");
	}

	base0  = renderer->addTexture("../Textures/Oxidated.jpg");
	bump0  = renderer->addTexture("../Textures/OxidatedBump.png", TEX_NORMALMAP);
	gloss0 = renderer->addTexture("../Textures/OxidatedGloss.png");

	/*base1  = renderer->addTexture("../Textures/iron02.jpg");
	bump1  = renderer->addTexture("../Textures/iron02Bump.jpg", TEX_NORMALMAP);
	gloss1 = renderer->addTexture("../Textures/iron02Gloss.jpg");
*/
	lightTexture = renderer->addParticleTexture(64);

	textTexture = renderer->addTexture("../Textures/font2.dds");


	Material *mat;

	mat = new Material();
	mat->setTextures(base0, bump0, gloss0);
    matTable.addMaterialName("Demwall", 256, 256, mat, 0);

/*	mat = new Material();
	mat->setTextures(base1, bump1, gloss1);
    matTable.addMaterialName("BrownFloor", 256, 256, mat, 0);
*/	
	T3dLoader t3dLoader;
	t3dLoader.loadFromFile("map.t3d", polys, matTable, &bumpPolyCreator, &polygonFilter, Vertex(0,0,0));

	/*for (int i = polys.getSize()-1; i >= 50; i--){
		delete polys[i];
		polys.remove(i);
	}*/
	fixTJunctions(polys);

	return true;
}

bool MainApp::unload(){
	int i, len = polys.getSize();

	for (i = 0; i < len; i++) delete polys[i];

	polys.clear();
	matTable.clear();
	return true;
}


void MainApp::drawWorld(){
	unsigned int i, j, k;

	for (i = 0; i < polys.getSize(); i++){
		((BumpPolygon *) polys[i])->clearDrawnFlag();
	}


	for (j = 0; j < sizeof(lights) / sizeof(Vertex); j++){

		renderer->changeRenderTarget(shadowCubemap);

		glMatrixMode(GL_PROJECTION);
		Matrix proj, mv;
		proj.loadProjectionMatrix(PI / 2, 1.0f, 1.0f, 3000.0f);
		glLoadMatrixf(proj);

		glDepthFunc(GL_LEQUAL);
		glEnable(GL_DEPTH_TEST);

		glEnable(GL_CULL_FACE);
		glCullFace(GL_BACK);
		glFrontFace(GL_CW);

		for (k = 0; k < 6; k++){
			renderer->changeRenderFace(shadowCubemap, k);
			
			//glClearColor(1,1,1,0);
			glClear(/*GL_COLOR_BUFFER_BIT | */GL_DEPTH_BUFFER_BIT);

			mv.loadCubemapMatrixOGL(k);
			glMatrixMode(GL_MODELVIEW);
			glLoadMatrixf(mv);
			glTranslatef(-lights[j].x, -lights[j].y, -lights[j].z);

			for (i = 0; i < polys.getSize(); i++){
				((BumpPolygon *) polys[i])->drawDepth(renderer, lights[j], 1800);
			}
		}

		renderer->changeRenderTarget();

		glProgramLocalParameter4fvARB(GL_FRAGMENT_PROGRAM_ARB, 1, cols[j]);

		for (i = 0; i < polys.getSize(); i++){
			((BumpPolygon *) polys[i])->draw(renderer, position, lights[j], 1800);
		}
	}

	for (i = 0; i < polys.getSize(); i++){
		((BumpPolygon *) polys[i])->drawAmbient(renderer);
	}
}


bool MainApp::drawFrame(){

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	modelView.loadRotateZXY(-Wx, -Wy, -Wz);
    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	modelView.translate(-position);
	glLoadMatrixf(modelView);

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);

	lights[0] = Vertex(600 * cosf(currTime * 0.53f),0,80);
	lights[1] = Vertex(330 * cosf(currTime) + 150 * sinf(currTime), 50 + 150 * cosf(0.63f * currTime) * sinf(0.42f * currTime), 330 * cosf(currTime) - 50 * sinf(currTime));
	cols[0] = Color(1,1,0);
	cols[1] = Color(0,1,1);


	drawWorld();

	Vertex dx(modelView[0], modelView[4], modelView[8]);
	Vertex dy(modelView[1], modelView[5], modelView[9]);

	// Draw lights
	renderer->setBlending(GL_ONE, GL_ONE);
	renderer->setTextures(lightTexture);
	renderer->apply();


	glDepthMask(GL_FALSE);
	glBegin(GL_QUADS);
	for (unsigned int i = 0; i < sizeof(lights) / sizeof(Vertex); i++){
		glColor3fv(cols[i]);

		glTexCoord2f(0,0);
		glVertex3fv(lights[i] + 30 * (-dx + dy));
		glTexCoord2f(1,0);
		glVertex3fv(lights[i] + 30 * ( dx + dy));
		glTexCoord2f(1,1);
		glVertex3fv(lights[i] + 30 * ( dx - dy));
		glTexCoord2f(0,1);
		glVertex3fv(lights[i] + 30 * (-dx - dy));
	}
	glEnd();
	glDepthMask(GL_TRUE);

	return true;
}

DECLARE_MAIN_APP(MainApp)
